@@ -199,10 +199,32 @@ class Face(topologic.Face):
         return baseFace
 
     @staticmethod
-    def CompassAngle(face, north, mantissa=4):
+    def CompassAngle(face, north: list = None, mantissa: int = 4):
+        """
+        Returns the horizontal compass angle in degrees between the normal vector of the input face and the input vector. The angle is measured in counter-clockwise fashion. Only the first two elements of the vectors are considered.
+
+        Parameters
+        ----------
+        face : topologic.Face
+            The input face.
+        north : list , optional
+            The second vector representing the north direction. The default is the positive YAxis ([0,1,0]).
+        mantissa : int, optional
+            The length of the desired mantissa. The default is 4.
+        tolerance : float , optional
+            The desired tolerance. The default is 0.0001.
+
+        Returns
+        -------
+        float
+            The horizontal compass angle in degrees between the direction of the face and the second input vector.
+
+        """
         from topologicpy.Vector import Vector
         if not isinstance(face, topologic.Face):
             return None
+        if not north:
+            north = Vector.North()
         dirA = Face.NormalAtParameters(face,mantissa=mantissa)
         return Vector.CompassAngle(vectorA=dirA, vectorB=north, mantissa=mantissa)
 
@@ -360,8 +382,6 @@ class Face(topologic.Face):
                 return topologic.Face.ByExternalBoundary(Wire.RemoveCollinearEdges(ext_boundary, angTolerance))
             except:
                 try:
-                    #w = Wire.RemoveCollinearEdges(ext_boundary, angTolerance)
-                    #print("Step 1 wire", w)
                     w = Wire.Planarize(ext_boundary)
                     f = Face.ByWire(w)
                     return f
@@ -416,10 +436,18 @@ class Face(topologic.Face):
 
         """
         from topologicpy.Topology import Topology
+        from topologicpy.Wire import Wire
 
         if not isinstance(vertices, list):
             return None
         vertexList = [x for x in vertices if isinstance(x, topologic.Vertex)]
+        if len(vertexList) < 3:
+            return None
+        
+        w = Wire.ByVertices(vertexList)
+        f = Face.ByExternalBoundary(w)
+        return f
+        """
         edges = []
         for i in range(len(vertexList)-1):
             v1 = vertexList[i]
@@ -442,7 +470,7 @@ class Face(topologic.Face):
             return topologic.Face.ByExternalBoundary(Topology.SelfMerge(topologic.Cluster.ByTopologies(edges, False)))
         else:
             return None
-
+        """
     def ByVerticesCluster(cluster):
         """
         Creates a face from the input cluster of vertices.
@@ -476,19 +504,68 @@ class Face(topologic.Face):
 
         Returns
         -------
-        topologic.Face
-            The created face.
+        topologic.Face or list
+            The created face. If the wire is non-planar, the method will attempt to triangulate the wire and return a list of faces.
 
         """
+        from topologicpy.Vertex import Vertex
         from topologicpy.Wire import Wire
+        from topologicpy.Shell import Shell
+        from topologicpy.Cluster import Cluster
         from topologicpy.Topology import Topology
+        from topologicpy.Dictionary import Dictionary
         import random
+
+        def triangulateWire(wire):
+            wire = Wire.RemoveCollinearEdges(wire)
+            vertices = Wire.Vertices(wire)
+            shell = Shell.Delaunay(vertices)
+            if isinstance(shell, topologic.Shell):
+                return Shell.Faces(shell)
+            else:
+                return []
         if not isinstance(wire, topologic.Wire):
             return None
         if not Wire.IsClosed(wire):
             return None
-        f = topologic.Face.ByExternalBoundary(wire)
-        return f
+        
+        edges = Wire.Edges(wire)
+        wire = Topology.SelfMerge(Cluster.ByTopologies(edges))
+        vertices = Wire.Vertices(wire)
+        #print("This wire has:", len(vertices), "vertices.")
+        try:
+            #print(Topology.IsPlanar(wire))
+            fList = topologic.Face.ByExternalBoundary(wire)
+        except:
+            if len(vertices) > 3:
+                print("This wire has:", len(vertices), "vertices.")
+                print("Non planar wire, triangulating")
+                fList = triangulateWire(wire)
+                print("After triangulation", fList)
+            else:
+                fList = []
+        
+        if not isinstance(fList, list):
+            fList = [fList]
+
+        returnList = []
+        for f in fList:
+            if Face.Area(f) < 0:
+                wire = Face.ExternalBoundary(f)
+                wire = Wire.Invert(wire)
+                try:
+                    f = topologic.Face.ByExternalBoundary(wire)
+                    returnList.append(f)
+                except:
+                    pass
+            else:
+                returnList.append(f)
+        if len(returnList) == 0:
+            return None
+        elif len(returnList) == 1:
+            return returnList[0]
+        else:
+            return returnList
 
     @staticmethod
     def ByWires(externalBoundary, internalBoundaries=[]):
@@ -722,6 +799,45 @@ class Face(topologic.Face):
             The flattened face.
 
         """
+
+        def leftMost(vertices, tolerance = 0.0001):
+            xCoords = []
+            for v in vertices:
+                xCoords.append(Vertex.Coordinates(vertices[0])[0])
+            minX = min(xCoords)
+            lmVertices = []
+            for v in vertices:
+                if abs(Vertex.Coordinates(vertices[0])[0] - minX) <= tolerance:
+                    lmVertices.append(v)
+            return lmVertices
+        
+        def bottomMost(vertices, tolerance = 0.0001):
+            yCoords = []
+            for v in vertices:
+                yCoords.append(Vertex.Coordinates(vertices[0])[1])
+            minY = min(yCoords)
+            bmVertices = []
+            for v in vertices:
+                if abs(Vertex.Coordinates(vertices[0])[1] - minY) <= tolerance:
+                    bmVertices.append(v)
+            return bmVertices
+
+        def vIndex(v, vList, tolerance):
+            for i in range(len(vList)):
+                if topologic.VertexUtility.Distance(v, vList[i]) < tolerance:
+                    return i+1
+            return None
+        
+        #  rotate cycle path such that it begins with the smallest node
+        def rotate_to_smallest(path):
+            n = path.index(min(path))
+            return path[n:]+path[:n]
+        
+        #  rotate vertices list so that it begins with the input vertex
+        def rotate_vertices(vertices, vertex):
+            n = vertices.index(vertex)
+            return vertices[n:]+vertices[:n]
+        
         from topologicpy.Vertex import Vertex
         from topologicpy.Topology import Topology
         from topologicpy.Dictionary import Dictionary
@@ -755,10 +871,14 @@ class Face(topologic.Face):
         flatFace = Topology.Rotate(flatFace, world_origin, 0, 1, 0, -theta)
         # Ensure flatness. Force Z to be zero
         flatExternalBoundary = Face.ExternalBoundary(flatFace)
-        flatFaceVertices = Wire.Vertices(flatExternalBoundary)
+        flatFaceVertices = Topology.SubTopologies(flatExternalBoundary, subTopologyType="vertex")
+    
         tempVertices = []
         for ffv in flatFaceVertices:
             tempVertices.append(Vertex.ByCoordinates(ffv.X(), ffv.Y(), 0))
+        
+        temp_v = bottomMost(leftMost(tempVertices))[0]
+        tempVertices = rotate_vertices(tempVertices, temp_v)
         flatExternalBoundary = Wire.ByVertices(tempVertices)
 
         internalBoundaries = Face.InternalBoundaries(flatFace)
@@ -768,6 +888,8 @@ class Face(topologic.Face):
             tempVertices = []
             for ibVertex in ibVertices:
                 tempVertices.append(Vertex.ByCoordinates(ibVertex.X(), ibVertex.Y(), 0))
+            temp_v = bottomMost(leftMost(tempVertices))[0]
+            tempVertices = rotate_vertices(tempVertices, temp_v)
             flatInternalBoundaries.append(Wire.ByVertices(tempVertices))
         flatFace = Face.ByWires(flatExternalBoundary, flatInternalBoundaries)
         dictionary = Dictionary.ByKeysValues(["xTran", "yTran", "zTran", "phi", "theta"], [cm.X(), cm.Y(), cm.Z(), phi, theta])
@@ -929,12 +1051,13 @@ class Face(topologic.Face):
 
         """
         from topologicpy.Wire import Wire
+
         if not isinstance(face, topologic.Face):
             return None
         eb = Face.ExternalBoundary(face)
         vertices = Wire.Vertices(eb)
-        reversed_vertices = vertices[::-1]
-        inverted_wire = Wire.ByVertices(reversed_vertices)
+        vertices.reverse()
+        inverted_wire = Wire.ByVertices(vertices)
         internal_boundaries = Face.InternalBoundaries(face)
         if not internal_boundaries:
             inverted_face = Face.ByWire(inverted_wire)
@@ -1465,8 +1588,8 @@ class Face(topologic.Face):
 
         """
         from topologicpy.Vertex import Vertex
+        from topologicpy.Wire import Wire
         from topologicpy.Topology import Topology
-        from topologicpy.Cluster import Cluster
         from topologicpy.Dictionary import Dictionary
 
         flatFace = Face.Flatten(face)
@@ -1494,7 +1617,10 @@ class Face(topologic.Face):
             f = Topology.Rotate(f, origin=world_origin, x=0, y=0, z=1, degree=phi)
             f = Topology.Translate(f, xTran, yTran, zTran)
             if Face.Angle(face, f) > 90:
-                finalFaces.append(Face.Invert(f))
+                wire = Face.ExternalBoundary(f)
+                wire = Wire.Invert(wire)
+                f = topologic.Face.ByExternalBoundary(wire)
+                finalFaces.append(f)
             else:
                 finalFaces.append(f)
         return finalFaces
