@@ -67,7 +67,6 @@ class Shell(Topology):
     def ByWires(wires, triangulate=True, tolerance=0.0001):
         """
         Creates a shell by lofting through the input wires
-
         Parameters
         ----------
         wires : list
@@ -76,12 +75,10 @@ class Shell(Topology):
             If set to True, the faces will be triangulated. The default is True.
         tolerance : float , optional
             The desired tolerance. The default is 0.0001.
-
         Returns
         -------
         topologic.Shell
             The creates shell.
-
         """
         from topologicpy.Edge import Edge
         from topologicpy.Wire import Wire
@@ -134,28 +131,28 @@ class Shell(Topology):
                     e3 = None
                     e4 = None
                     try:
-                        e3 = topologic.Edge.ByStartVertexEndVertex(e1.StartVertex(), e2.StartVertex())
+                        e3 = Edge.ByVertices([e1.StartVertex(), e2.StartVertex()])
                     except:
                         try:
-                            e4 = topologic.Edge.ByStartVertexEndVertex(e1.EndVertex(), e2.EndVertex())
+                            e4 = Edge.ByVertices([e1.EndVertex(), e2.EndVertex()])
                         except:
                             pass
                     try:
-                        e4 = topologic.Edge.ByStartVertexEndVertex(e1.EndVertex(), e2.EndVertex())
+                        e4 = Edge.ByVertices([e1.EndVertex(), e2.EndVertex()])
                     except:
                         try:
-                            e3 = topologic.Edge.ByStartVertexEndVertex(e1.StartVertex(), e2.StartVertex())
+                            e3 = Edge.ByVertices([e1.StartVertex(), e2.StartVertex()])
                         except:
                             pass
                     if e3 and e4:
                         try:
-                            faces.append(topologic.Face.ByExternalBoundary(topologic.Wire.ByEdges([e1, e4, e2, e3])))
+                            faces.append(Face.ByWire(topologic.Wire.ByEdges([e1, e4, e2, e3])))
                         except:
-                            faces.append(topologic.Face.ByExternalBoundary(topologic.Wire.ByEdges([e1, e3, e2, e4])))
+                            faces.append(Face.ByWire(topologic.Wire.ByEdges([e1, e3, e2, e4])))
                     elif e3:
-                            faces.append(topologic.Face.ByExternalBoundary(topologic.Wire.ByEdges([e1, e3, e2])))
+                            faces.append(Face.ByWire(topologic.Wire.ByEdges([e1, e3, e2])))
                     elif e4:
-                            faces.append(topologic.Face.ByExternalBoundary(topologic.Wire.ByEdges([e1, e4, e2])))
+                            faces.append(Face.ByWire(topologic.Wire.ByEdges([e1, e4, e2])))
         return Shell.ByFaces(faces, tolerance)
 
     @staticmethod
@@ -178,12 +175,12 @@ class Shell(Topology):
             The creates shell.
 
         """
+        from topologicpy.Cluster import Cluster
         if not cluster:
             return None
         if not isinstance(cluster, topologic.Cluster):
             return None
-        wires = []
-        _ = cluster.Wires(None, wires)
+        wires = Cluster.Wires(cluster)
         return Shell.ByWires(wires, triangulate=triangulate, tolerance=tolerance)
 
     @staticmethod
@@ -269,6 +266,9 @@ class Shell(Topology):
         phi = Dictionary.ValueAtKey(dictionary,"phi")
         theta = Dictionary.ValueAtKey(dictionary,"theta")
 
+        print("Delaunay", len(vertices))
+        print("Dealunay", Dictionary.Keys(dictionary))
+        print("Delaunay", Dictionary.Values(dictionary))
         # Create a Vertex at the world's origin (0,0,0)
         world_origin = Vertex.ByCoordinates(0,0,0)
 
@@ -299,7 +299,6 @@ class Shell(Topology):
             faces.append(Face.ByWire(Wire.ByVertices(tempTriangleVertices)))
 
         shell = Shell.ByFaces(faces)
-        print(shell)
         if isinstance(face, topologic.Face):
             edges = Shell.Edges(shell)
             edgesCluster = Cluster.ByTopologies(edges)
@@ -387,7 +386,96 @@ class Shell(Topology):
         faces = []
         _ = shell.Faces(None, faces)
         return faces
+    
+    @staticmethod
+    def IsInside(shell: topologic.Shell, vertex: topologic.Vertex, tolerance: float = 0.0001) -> bool:
+        """
+        Returns True if the input vertex is inside the input shell. Returns False otherwise. Inside is defined as being inside one of the shell's faces
 
+        Parameters
+        ----------
+        shell : topologic.Shell
+            The input shell.
+        vertex : topologic.Vertex
+            The input vertex.
+        tolerance : float , optional
+            The desired tolerance. The default is 0.0001.
+
+        Returns
+        -------
+        bool
+            Returns True if the input vertex is inside the input shell. Returns False otherwise.
+
+        """
+
+        from topologicpy.Face import Face
+        if not isinstance(shell, topologic.Shell):
+            return None
+        if not isinstance(vertex, topologic.Vertex):
+            return None
+        faces = Shell.Faces(shell)
+        for f in faces:
+            if Face.IsInside(fface=f, vertex=vertex, tolerance=tolerance):
+                return True
+        return False
+    
+    @staticmethod
+    def IsOnBoundary(shell: topologic.Shell, vertex: topologic.Vertex, tolerance: float = 0.0001) -> bool:
+        """
+        Returns True if the input vertex is inside the input shell. Returns False otherwise. Inside is defined as being inside one of the shell's faces
+
+        Parameters
+        ----------
+        shell : topologic.Shell
+            The input shell.
+        vertex : topologic.Vertex
+            The input vertex.
+        tolerance : float , optional
+            The desired tolerance. The default is 0.0001.
+
+        Returns
+        -------
+        bool
+            Returns True if the input vertex is inside the input shell. Returns False otherwise.
+
+        """
+
+        from topologicpy.Wire import Wire
+
+        if not isinstance(shell, topologic.Shell):
+            return None
+        if not isinstance(vertex, topologic.Vertex):
+            return None
+        boundary = Shell.ExternalBoundary(shell)
+        return Wire.IsInside(wire=boundary, vertex=vertex, tolerance=tolerance)
+    
+    @staticmethod
+    def IsOutside(shell: topologic.Shell, vertex: topologic.Vertex, tolerance: float = 0.0001) -> bool:
+        """
+        Returns True if the input vertex is inside the input shell. Returns False otherwise. Inside is defined as being inside one of the shell's faces
+
+        Parameters
+        ----------
+        shell : topologic.Shell
+            The input shell.
+        vertex : topologic.Vertex
+            The input vertex.
+        tolerance : float , optional
+            The desired tolerance. The default is 0.0001.
+
+        Returns
+        -------
+        bool
+            Returns True if the input vertex is inside the input shell. Returns False otherwise.
+
+        """
+
+        if not isinstance(shell, topologic.Shell):
+            return None
+        if not isinstance(vertex, topologic.Vertex):
+            return None
+        return not Wire.IsInside(shell=shell, vertex=vertex, tolerance=tolerance)
+    
     @staticmethod
     def HyperbolicParaboloidRectangularDomain(origin=None, llVertex=None, lrVertex=None, ulVertex=None, urVertex=None, u=10, v=10, direction=[0,0,1], placement="bottom"):
         """
@@ -902,7 +990,7 @@ class Shell(Topology):
         for i in range(uSides):
             for j in range(vSides):
                 rOrigin = Vertex.ByCoordinates(i*uOffset - wOffset, j*vOffset - lOffset, 0)
-                w = Wire.Rectangle(origin=rOrigin, width=uOffset, length=vOffset, placement="lowerleft", tolerance=tolerance)
+                w = Wire.Rectangle(origin=rOrigin, width=uOffset, length=vOffset, direction=[0,0,1], placement="lowerleft", tolerance=tolerance)
                 f = Face.ByWire(w)
                 faces.append(f)
         shell = Shell.ByFaces(faces)
